#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Networks;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Linq;

namespace HeuristicLab.Operators {
  [Item("HookOperator", "An operator which interacts with an optimization network port.")]
  [StorableClass]
  public class HookOperator : Operator, IHookOperator {
    public new ParameterCollection Parameters {
      get { return base.Parameters; }
    }

    [Storable]
    protected IGenericPort port;
    public IGenericPort Port {
      get { return port; }
      set {
        if (port != value) {
          port = value;
          OnPortChanged();
        }
      }
    }

    [StorableConstructor]
    protected HookOperator(bool deserializing) : base(deserializing) { }
    protected HookOperator(HookOperator original, Cloner cloner)
      : base(original, cloner) {
      // ATTENTION: port is intentionally NOT cloned
      port = original.port;
    }
    public HookOperator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new HookOperator(this, cloner);
    }

    public event EventHandler PortChanged;
    protected virtual void OnPortChanged() {
      var handler = PortChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public override IOperation Apply() {
      if (Port != null) {
        var msg = Port.PrepareMessage();
        foreach (var output in Port.Parameters.Where(p => p.Type.HasFlag(PortParameterType.Output))) {
          IParameter param;
          if (Parameters.TryGetValue(output.Name, out param)) {
            msg[output.Name] = param.ActualValue;
          }
        }

        Port.SendMessage(msg, CancellationToken);

        foreach (var input in Port.Parameters.Where(p => p.Type.HasFlag(PortParameterType.Input))) {
          IParameter param;
          if (Parameters.TryGetValue(input.Name, out param)) {
            param.ActualValue = msg[input.Name];
          }
        }
      }
      return null;
    }
  }
}
